@@ -83,8 +83,8 @@ mkdata.lognorm <- function(y,eta,wt,offset,nu)
     }
     xx <- nu[[1]]*(log(xx)-eta)
     zz <- nu[[1]]*(log(zz)-eta)
-    s.xx <- ifelse(xx<7,dnorm(xx)/(1-pnorm(xx)),xx+.15)
-    s.zz <- ifelse(zz<7,dnorm(zz)/(1-pnorm(zz)),zz+.15)
+    s.xx <- ifelse(xx<7,dnorm(xx)/(1-pnorm(xx)),xx+1/xx)
+    s.zz <- ifelse(zz<7,dnorm(zz)/(1-pnorm(zz)),zz+1/zz)
     s.xx <- pmax(s.xx,s.zz)
     u <- nu[[1]]*(delta*(s.xx-xx)-(s.xx-s.zz))
     w <- (s.xx^2/2-xx*s.xx+xx^2/2+log(s.xx)+log(2*pi)/2)
